@@ -1,7 +1,10 @@
 #ifndef ARQIO_H
 #define ARQIO_H
 
-class Socket;
+#include <vector>
+
+#include "threads.h"
+#include "socket.h"
 
 extern void WriteARQsocket(unsigned char* data, size_t len);
 extern bool Socket_arqRx();
