@@ -61,8 +61,10 @@ enum {
 	MODE_DOMINOEX11,
 	MODE_DOMINOEX16,
 	MODE_DOMINOEX22,
+	MODE_DOMINOEX44,
+	MODE_DOMINOEX88,
 	MODE_DOMINOEX_FIRST = MODE_DOMINOEX4,
-	MODE_DOMINOEX_LAST = MODE_DOMINOEX22,
+	MODE_DOMINOEX_LAST = MODE_DOMINOEX88,
 
 	MODE_FELDHELL,
 	MODE_SLOWHELL,
@@ -77,14 +79,14 @@ enum {
 	MODE_MFSK8,
 	MODE_MFSK16,
 	MODE_MFSK32,
-// experimental modes
 	MODE_MFSK4,
 	MODE_MFSK11,
 	MODE_MFSK22,
 	MODE_MFSK31,
 	MODE_MFSK64,
+	MODE_MFSK128,
 	MODE_MFSK_FIRST = MODE_MFSK8,
-	MODE_MFSK_LAST = MODE_MFSK64,
+	MODE_MFSK_LAST = MODE_MFSK128,
 
 	MODE_WEFAX_576,
 	MODE_WEFAX_288,
@@ -113,13 +115,25 @@ enum {
 	MODE_QPSK125,
 	MODE_QPSK250,
 	MODE_QPSK500,
+
 	MODE_PSK125R,
 	MODE_PSK250R,
 	MODE_PSK500R,
+	MODE_PSK1000,
+	MODE_PSK1000R,
 	MODE_PSK_FIRST = MODE_PSK31,
-	MODE_PSK_LAST = MODE_PSK500R,
+	MODE_PSK_LAST = MODE_PSK1000R,
 
 	MODE_OLIVIA,
+	MODE_OLIVIA_4_250,
+	MODE_OLIVIA_8_250,
+	MODE_OLIVIA_4_500,
+	MODE_OLIVIA_8_500,
+	MODE_OLIVIA_16_500,
+	MODE_OLIVIA_8_1000,
+	MODE_OLIVIA_16_1000,
+	MODE_OLIVIA_32_1000,
+	MODE_OLIVIA_64_2000,
 
 	MODE_RTTY,
 
@@ -127,11 +141,14 @@ enum {
 	MODE_THOR5,
 	MODE_THOR8,
 	MODE_THOR11,
-//	MODE_TSOR11,
 	MODE_THOR16,
 	MODE_THOR22,
+	MODE_THOR25x4,
+	MODE_THOR50x1,
+	MODE_THOR50x2,
+	MODE_THOR100,
 	MODE_THOR_FIRST = MODE_THOR4,
-	MODE_THOR_LAST = MODE_THOR22,
+	MODE_THOR_LAST = MODE_THOR100,
 
 	MODE_THROB1,
 	MODE_THROB2,
@@ -142,6 +159,42 @@ enum {
 	MODE_THROB_FIRST = MODE_THROB1,
 	MODE_THROB_LAST = MODE_THROBX4,
 
+// high speed && multiple carrier modes
+
+	MODE_4X_PSK63R,
+	MODE_5X_PSK63R,
+	MODE_10X_PSK63R,
+	MODE_20X_PSK63R,
+	MODE_32X_PSK63R,
+
+	MODE_4X_PSK125R,
+	MODE_5X_PSK125R,
+	MODE_10X_PSK125R,
+	MODE_12X_PSK125,
+	MODE_12X_PSK125R,
+	MODE_16X_PSK125R,
+
+	MODE_6X_PSK250,
+
+	MODE_2X_PSK250R,
+	MODE_3X_PSK250R,
+	MODE_5X_PSK250R,
+	MODE_6X_PSK250R,
+	MODE_7X_PSK250R,
+
+	MODE_2X_PSK500,
+	MODE_4X_PSK500,
+
+	MODE_2X_PSK500R,
+	MODE_3X_PSK500R,
+	MODE_4X_PSK500R,
+
+	MODE_2X_PSK800,
+	MODE_2X_PSK800R,
+
+	MODE_2X_PSK1000,
+	MODE_2X_PSK1000R,
+
 	MODE_SSB,
 	MODE_WWV,
 	MODE_ANALYSIS,
