@@ -41,7 +41,6 @@
 	#include "rigclass.h"
 #endif
 
-#include "rigMEM.h"
 #include "rigio.h"
 #include "rigxml.h"
 #include "debug.h"
@@ -440,7 +439,6 @@ void configuration::loadDefaults()
 	chkPseudoFSK->value(PseudoFSK);
 	chkUOSrx->value(UOSrx);
 	chkUOStx->value(UOStx);
-	chkXagc->value(Xagc);
 
 	mnuRTTYAFCSpeed->value(rtty_afcspeed);
 	btnPreferXhairScope->value(PreferXhairScope);
@@ -531,6 +529,9 @@ int configuration::setDefaults()
 	valTHOR_FILTER->value(THOR_FILTER);
 	valTHOR_PATHS->value(THOR_PATHS);
 	valThorCWI->value(ThorCWI);
+	valTHOR_PREAMBLE->value(THOR_PREAMBLE);
+	valTHOR_SOFTSYMBOLS->value(THOR_SOFTSYMBOLS);
+	valTHOR_SOFTBITS->value(THOR_SOFTBITS);
 
 	valDominoEX_BW->value(DOMINOEX_BW);
 	valDominoEX_FILTER->value(DOMINOEX_FILTER);
@@ -538,7 +539,6 @@ int configuration::setDefaults()
 	valDominoEX_PATHS->value(DOMINOEX_PATHS);
 	valDomCWI->value(DomCWI);
 
-	btnMEMMAPptt->value(MEMMAPptt);
 	btnRigCatCMDptt->value(RigCatCMDptt);
 	btnTTYptt->value(TTYptt);
 	btnUsePPortPTT->value(progdefaults.UsePPortPTT);
@@ -564,21 +564,20 @@ int configuration::setDefaults()
 
 	inpTTYdev->value(PTTdev.c_str());
 
-	if(chkUSEMEMMAPis) {
-		chkUSEMEMMAP->value(1); 
-		chkUSEHAMLIB->value(0); chkUSERIGCAT->value(0); chkUSEXMLRPC->value(0);
-	} else if (chkUSEHAMLIBis) {
+	if (chkUSEHAMLIBis) {
 		chkUSEHAMLIB->value(1);
-		chkUSEMEMMAP->value(0); chkUSERIGCAT->value(0);  chkUSEXMLRPC->value(0);
+		chkUSERIGCAT->value(0);  chkUSEXMLRPC->value(0);
 	} else if (chkUSERIGCATis) {
 		chkUSERIGCAT->value(1);
-		chkUSEMEMMAP->value(0); chkUSEHAMLIB->value(0); chkUSEXMLRPC->value(0);
+		chkUSEHAMLIB->value(0); chkUSEXMLRPC->value(0);
 	} else if (chkUSEXMLRPCis) {
 		chkUSEXMLRPC->value(1);
-		chkUSEMEMMAP->value(0); chkUSEHAMLIB->value(0); chkUSERIGCAT->value(0);
+		chkUSEHAMLIB->value(0); chkUSERIGCAT->value(0);
 	} else {
-		chkUSEMEMMAP->value(0); chkUSEHAMLIB->value(0); 
-		chkUSERIGCAT->value(0);	chkUSEHAMLIB->value(0); chkUSEXMLRPC->value(0);
+		chkUSEHAMLIB->value(0); 
+		chkUSERIGCAT->value(0);
+		chkUSEHAMLIB->value(0);
+		chkUSEXMLRPC->value(0);
 	}
 	if (!XmlRigFilename.empty()) readRigXML();
 
@@ -766,14 +765,9 @@ void configuration::initInterface()
 // close down any possible rig interface threads
 #if USE_HAMLIB
 	hamlib_close();
-//		MilliSleep(100);
 #endif
-	rigMEM_close();
-//		MilliSleep(100);
 	rigCAT_close();
-//		MilliSleep(100);
 
-	MEMMAPptt = btnMEMMAPptt->value();
 	RigCatCMDptt = btnRigCatCMDptt->value();
 	TTYptt = btnTTYptt->value();
 
@@ -788,7 +782,6 @@ void configuration::initInterface()
 	chkUSEHAMLIBis = chkUSEHAMLIB->value();
      HamlibCMDptt = btnHamlibCMDptt->value();
 #endif
-	chkUSEMEMMAPis = chkUSEMEMMAP->value();
 	chkUSERIGCATis = chkUSERIGCAT->value();
 
 #if USE_HAMLIB
@@ -806,13 +799,9 @@ void configuration::initInterface()
 
 	bool riginitOK = false;
 
-	if (chkUSEMEMMAPis) {// start the memory mapped i/o thread
-		if (rigMEM_init()) {
-			wf->setQSY(1);
-			riginitOK = true;
-		}
-	} else if (chkUSERIGCATis) { // start the rigCAT thread
+	if (chkUSERIGCATis) { // start the rigCAT thread
 		if (rigCAT_init(true)) {
+			LOG_VERBOSE("%s", "using rigCAT");
 			wf->USB(true);
 			wf->setQSY(1);
 			riginitOK = true;
@@ -827,13 +816,16 @@ void configuration::initInterface()
 #endif
 	} else if (chkUSEXMLRPCis) {
 		wf->setXMLRPC(1);
-		rigCAT_init(false);
-		wf->USB(true);
-		wf->setQSY(0);
-		riginitOK = true;
+		if (rigCAT_init(false)) {
+			LOG_VERBOSE("%s", "using XMLRPC");
+			wf->USB(true);
+			wf->setQSY(0);
+			riginitOK = true;
+		}
 	}
 
 	if (riginitOK == false) {
+		LOG_VERBOSE("%s", "NO rig control");
 		rigCAT_init(false);
 		wf->USB(true);
 		wf->setQSY(0);
@@ -841,8 +833,6 @@ void configuration::initInterface()
 
 	if (HamlibCMDptt && chkUSEHAMLIBis)
 		push2talk->reset(PTT::PTT_HAMLIB);
-	else if (MEMMAPptt && chkUSEMEMMAPis)
-		push2talk->reset(PTT::PTT_MEMMAP);
 	else if ((RigCatCMDptt || RigCatRTSptt || RigCatDTRptt) && chkUSERIGCATis)
 		push2talk->reset(PTT::PTT_RIGCAT);
 	else if (TTYptt)
