@@ -102,9 +102,12 @@ void loadBrowser(Fl_Widget *widget) {
 #endif
 
 	w->add(LINE_SEP);
-	w->add(_("<LOG>\tsave QSO data"));
-	w->add(_("<LNW>\tlog at xmt time"));
 	w->add(_("<CLRLOG>\tclear log fields"));
+	w->add(_("<LOG>\tsave QSO data"));
+	w->add(_("<LOG:msg>\tsaveQSO data, append msg to notes"));
+	w->add(_("<LNW>\tlog at xmt time"));
+	w->add(_("<LNW:msg>\tsaveQSO data, append msg to notes"));
+	w->add(_("<EQSL>\tlog eQSL"));
 	w->add(_("<EQSL:[msg]>\tlog eQSL optional msg"));
 
 	w->add(LINE_SEP);
@@ -141,6 +144,9 @@ void loadBrowser(Fl_Widget *widget) {
 	w->add(_("<QSYTO>\tleft-clk QSY button"));
 	w->add(_("<QSYFM>\tright-clk QSY button"));
 
+	w->add(LINE_SEP);
+	w->add(_("<QRG:text>\tinsert QRG into Rx text"));
+
 	w->add(LINE_SEP);
 	w->add(_("<QSY:FFF.F[:NNNN]>\tqsy to kHz, Hz"));
 	w->add(_("<RIGMODE:mode>\tvalid xcvr mode"));
@@ -148,6 +154,9 @@ void loadBrowser(Fl_Widget *widget) {
 
 	w->add(LINE_SEP);
 	w->add(_("<FILE:>\tinsert text file"));
+	w->add(LINE_SEP);
+
+	w->add(_("<PAUSE>\tpause transmit"));
 	w->add(_("<IDLE:NN.nn>\tidle signal for NN.nn sec"));
 	w->add(_("<TIMER:NN>\trepeat every NN sec"));
 	w->add(_("<TUNE:NN>\ttune signal for NN sec"));
@@ -155,6 +164,9 @@ void loadBrowser(Fl_Widget *widget) {
 	w->add(_("<REPEAT>\trepeat macro continuously"));
 	w->add(_("<SKED:hhmm[:YYYYDDMM]>\tschedule execution"));
 
+	w->add(LINE_SEP);
+	w->add(_("<TXATTEN:nn.n>\t set xmt attenuator"));
+
 	w->add(LINE_SEP);
 	w->add(_("<CWID>\tCW identifier"));
 	w->add(_("<ID>\tsend mode ID in video text"));
@@ -177,6 +189,10 @@ void loadBrowser(Fl_Widget *widget) {
 
 	w->add(LINE_SEP);
 	w->add(_("<MACROS:>\tchange macro defs file"));
+	w->add(_("<SAVE>\tsave current macro file"));
+
+	w->add(LINE_SEP);
+	w->add(_("<COMMENT:text>\tuntransmitted comment"));
 
 	w->add(LINE_SEP);
 	assert(MODE_CONTESTIA < MODE_OLIVIA);
