@@ -65,7 +65,6 @@
 #if USE_HAMLIB
         #include "hamlib.h"
 #endif
-#include "rigMEM.h"
 #include "rigio.h"
 #include "debug.h"
 #include "re.h"
@@ -243,7 +242,7 @@ static void set_combo_value(Fl_ComboBox* box, const string& s)
 
 static void get_combo_contents(Fl_ComboBox* box, vector<xmlrpc_c::value>* items)
 {
-	int n = box->size(), p = box->index();
+	int n = box->lsize(), p = box->index();
 	items->reserve(n);
 	for (int i = 0; i < n; i++) {
 		box->index(i);
@@ -1682,8 +1681,9 @@ static void set_rig_control(bool xmlrpc)
 
 	if (xmlrpc) {
 		rcb_t b[] = {
-			{ chkUSERIGCAT, btnInitRIGCAT }, { chkUSEHAMLIB, btnInitHAMLIB },
-			{ chkUSEMEMMAP, btnInitMEMMAP }, { chkUSEXMLRPC, btnInitXMLRPC }
+			{ chkUSERIGCAT, btnInitRIGCAT },
+			{ chkUSEHAMLIB, btnInitHAMLIB },
+			{ chkUSEXMLRPC, btnInitXMLRPC }
 		};
 		for (size_t i = 0; i < sizeof(b)/sizeof(*b); i++) {
 			if (b[i].toggle->value()) {
