@@ -76,12 +76,12 @@ int mt63::tx_process()
     }
 
 	c = get_tx_char();
-	if (c == 0x03)  {
+	if (c == GET_TX_CHAR_ETX)  {
 		stopflag = true;
 		flush = Tx->DataInterleave;
 	}
 
-	if (c == -1 || stopflag == true) c = 0;
+	if (c == GET_TX_CHAR_NODATA || stopflag == true) c = 0;
 
 	if (stopflag) {
 		stopflag = false;
@@ -106,70 +106,33 @@ int mt63::tx_process()
 		return -1;	/* we're done */
 	}
 
-	if ((progdefaults.mt63_8bit && c > 255) || (!progdefaults.mt63_8bit && c > 127))
+	if (c > 255 || (!progdefaults.mt63_8bit && c > 127))
 		c = '.';
+	
+	int sendc = c;
+	
+	if (sendc > 127) {
+		sendc &= 127;
+		Tx->SendChar(127);
+		for (int i = 0; i < Tx->Comb.Output.Len; i++)
+			if (fabs(Tx->Comb.Output.Data[i]) > maxval)
+				maxval = fabs(Tx->Comb.Output.Data[i]);
+		for (int i = 0; i < Tx->Comb.Output.Len; i++)
+			Tx->Comb.Output.Data[i] /= maxval;
+		ModulateXmtr((Tx->Comb.Output.Data), Tx->Comb.Output.Len);
+	}
+
+	Tx->SendChar(sendc);
+	for (int i = 0; i < Tx->Comb.Output.Len; i++)
+		if (fabs(Tx->Comb.Output.Data[i]) > maxval)
+			maxval = fabs(Tx->Comb.Output.Data[i]);
+	for (int i = 0; i < Tx->Comb.Output.Len; i++) {
+		Tx->Comb.Output.Data[i] /= maxval;
+	}
+	ModulateXmtr((Tx->Comb.Output.Data), Tx->Comb.Output.Len);
 
 	put_echo_char(c);
 
-	if (c & 0xFF00) { // UTF-8 character send two bytes
-		unsigned char c1 = (c >> 8) & 0xFF;
-		unsigned char c2 = (c & 0xFF);
-		if (c1 > 127) {
-			c1 &= 127;
-			Tx->SendChar(127);
-			for (int i = 0; i < Tx->Comb.Output.Len; i++)
-				if (fabs(Tx->Comb.Output.Data[i]) > maxval)
-					maxval = fabs(Tx->Comb.Output.Data[i]);
-			for (int i = 0; i < Tx->Comb.Output.Len; i++)
-				Tx->Comb.Output.Data[i] /= maxval;
-			ModulateXmtr((Tx->Comb.Output.Data), Tx->Comb.Output.Len);
-		}
-		Tx->SendChar(c1);
-		for (int i = 0; i < Tx->Comb.Output.Len; i++)
-			if (fabs(Tx->Comb.Output.Data[i]) > maxval)
-				maxval = fabs(Tx->Comb.Output.Data[i]);
-		for (int i = 0; i < Tx->Comb.Output.Len; i++) {
-			Tx->Comb.Output.Data[i] /= maxval;
-		}
-		ModulateXmtr((Tx->Comb.Output.Data), Tx->Comb.Output.Len);
-		if (c2 > 127) {
-			c2 &= 127;
-			Tx->SendChar(127);
-			for (int i = 0; i < Tx->Comb.Output.Len; i++)
-				if (fabs(Tx->Comb.Output.Data[i]) > maxval)
-					maxval = fabs(Tx->Comb.Output.Data[i]);
-			for (int i = 0; i < Tx->Comb.Output.Len; i++)
-				Tx->Comb.Output.Data[i] /= maxval;
-			ModulateXmtr((Tx->Comb.Output.Data), Tx->Comb.Output.Len);
-		}
-		Tx->SendChar(c2);
-		for (int i = 0; i < Tx->Comb.Output.Len; i++)
-			if (fabs(Tx->Comb.Output.Data[i]) > maxval)
-				maxval = fabs(Tx->Comb.Output.Data[i]);
-		for (int i = 0; i < Tx->Comb.Output.Len; i++) {
-			Tx->Comb.Output.Data[i] /= maxval;
-		}
-		ModulateXmtr((Tx->Comb.Output.Data), Tx->Comb.Output.Len);
-	} else {
-		if (c > 127) {
-			c &= 127;
-			Tx->SendChar(127);
-			for (int i = 0; i < Tx->Comb.Output.Len; i++)
-				if (fabs(Tx->Comb.Output.Data[i]) > maxval)
-					maxval = fabs(Tx->Comb.Output.Data[i]);
-			for (int i = 0; i < Tx->Comb.Output.Len; i++)
-				Tx->Comb.Output.Data[i] /= maxval;
-			ModulateXmtr((Tx->Comb.Output.Data), Tx->Comb.Output.Len);
-		}
-		Tx->SendChar(c);
-		for (int i = 0; i < Tx->Comb.Output.Len; i++)
-			if (fabs(Tx->Comb.Output.Data[i]) > maxval)
-				maxval = fabs(Tx->Comb.Output.Data[i]);
-		for (int i = 0; i < Tx->Comb.Output.Len; i++) {
-			Tx->Comb.Output.Data[i] /= maxval;
-		}
-		ModulateXmtr((Tx->Comb.Output.Data), Tx->Comb.Output.Len);
-	}
 	return 0;
 }
 
