@@ -55,7 +55,6 @@
 #if USE_HAMLIB
 	#include "hamlib.h"
 #endif
-#include "rigMEM.h"
 #include "rigio.h"
 
 #include "fldigi-config.h"
@@ -1522,10 +1521,8 @@ void waterfall::Carrier(int f)
 	active_modem->set_freq(f);
 }
 
-extern void viewer_redraw();
 void waterfall::rfcarrier(long long cf) {
 	wfdisp->rfcarrier(cf);
-//	REQ(&viewer_redraw);
 }
 
 long long waterfall::rfcarrier() {
